"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.commandNames = exports.shorthands = exports.cliOptionsTypes = exports.rcOptionsTypes = void 0;
exports.help = help;
exports.handler = handler;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const cli_meta_1 = require("@pnpm/cli-meta");
const cli_utils_1 = require("@pnpm/cli-utils");
const logger_1 = require("@pnpm/logger");
const os_env_path_extender_1 = require("@pnpm/os.env.path-extender");
const render_help_1 = __importDefault(require("render-help"));
const rimraf_1 = __importDefault(require("@zkochan/rimraf"));
const rcOptionsTypes = () => ({});
exports.rcOptionsTypes = rcOptionsTypes;
const cliOptionsTypes = () => ({
    force: Boolean,
});
exports.cliOptionsTypes = cliOptionsTypes;
exports.shorthands = {};
exports.commandNames = ['setup'];
function help() {
    return (0, render_help_1.default)({
        description: 'Sets up pnpm',
        descriptionLists: [
            {
                title: 'Options',
                list: [
                    {
                        description: 'Override the PNPM_HOME env variable in case it already exists',
                        name: '--force',
                        shortAlias: '-f',
                    },
                ],
            },
        ],
        url: (0, cli_utils_1.docsUrl)('setup'),
        usages: ['pnpm setup'],
    });
}
function getExecPath() {
    if ((0, cli_meta_1.detectIfCurrentPkgIsExecutable)()) {
        // If the pnpm CLI was bundled by vercel/pkg then we cannot use the js path for npm_execpath
        // because in that case the js is in a virtual filesystem inside the executor.
        // Instead, we use the path to the exe file.
        return process.execPath;
    }
    return (require.main != null) ? require.main.filename : process.cwd();
}
function copyCli(currentLocation, targetDir) {
    const newExecPath = path_1.default.join(targetDir, path_1.default.basename(currentLocation));
    if (path_1.default.relative(newExecPath, currentLocation) === '')
        return;
    logger_1.logger.info({
        message: `Copying pnpm CLI from ${currentLocation} to ${newExecPath}`,
        prefix: process.cwd(),
    });
    fs_1.default.mkdirSync(targetDir, { recursive: true });
    rimraf_1.default.sync(newExecPath);
    fs_1.default.copyFileSync(currentLocation, newExecPath);
}
function createPnpxScripts(targetDir) {
    // Why script files instead of aliases?
    // 1. Aliases wouldn't work on all platform, such as Windows Command Prompt or POSIX `sh`.
    // 2. Aliases wouldn't work on all environments, such as non-interactive shells and CI environments.
    // 3. Aliases must be set for different shells while script files are limited to only 2 types: POSIX and Windows.
    // 4. Aliases cannot be located with the `which` or `where` command.
    // 5. Editing rc files is more error-prone than just write new files to the filesystem.
    fs_1.default.mkdirSync(targetDir, { recursive: true });
    // windows can also use shell script via mingw or cygwin so no filter
    const shellScript = [
        '#!/bin/sh',
        'exec pnpm dlx "$@"',
    ].join('\n');
    fs_1.default.writeFileSync(path_1.default.join(targetDir, 'pnpx'), shellScript, { mode: 0o755 });
    if (process.platform === 'win32') {
        const batchScript = [
            '@echo off',
            'pnpm dlx %*',
        ].join('\n');
        fs_1.default.writeFileSync(path_1.default.join(targetDir, 'pnpx.cmd'), batchScript);
        const powershellScript = 'pnpm dlx @args';
        fs_1.default.writeFileSync(path_1.default.join(targetDir, 'pnpx.ps1'), powershellScript);
    }
}
async function handler(opts) {
    const execPath = getExecPath();
    if (execPath.match(/\.[cm]?js$/) == null) {
        copyCli(execPath, opts.pnpmHomeDir);
        createPnpxScripts(opts.pnpmHomeDir);
    }
    try {
        const report = await (0, os_env_path_extender_1.addDirToEnvPath)(opts.pnpmHomeDir, {
            configSectionName: 'pnpm',
            proxyVarName: 'PNPM_HOME',
            overwrite: opts.force,
            position: 'start',
        });
        return renderSetupOutput(report);
    }
    catch (err) { // eslint-disable-line
        switch (err.code) {
            case 'ERR_PNPM_BAD_ENV_FOUND':
                err.hint = 'If you want to override the existing env variable, use the --force option';
                break;
            case 'ERR_PNPM_BAD_SHELL_SECTION':
                err.hint = 'If you want to override the existing configuration section, use the --force option';
                break;
        }
        throw err;
    }
}
function renderSetupOutput(report) {
    if (report.oldSettings === report.newSettings) {
        return 'No changes to the environment were made. Everything is already up to date.';
    }
    const output = [];
    if (report.configFile) {
        output.push(reportConfigChange(report.configFile));
    }
    output.push(`Next configuration changes were made:
${report.newSettings}`);
    if (report.configFile == null) {
        output.push('Setup complete. Open a new terminal to start using pnpm.');
    }
    else if (report.configFile.changeType !== 'skipped') {
        output.push(`To start using pnpm, run:
source ${report.configFile.path}
`);
    }
    return output.join('\n\n');
}
function reportConfigChange(configReport) {
    switch (configReport.changeType) {
        case 'created': return `Created ${configReport.path}`;
        case 'appended': return `Appended new lines to ${configReport.path}`;
        case 'modified': return `Replaced configuration in ${configReport.path}`;
        case 'skipped': return `Configuration already up to date in ${configReport.path}`;
    }
}
//# sourceMappingURL=setup.js.map